@@ -10,54 +10,58 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "lang/lang_keys.h"
 #include "storage/localstorage.h"
 #include "storage/storage_media_prepare.h"
+#include "iv/iv_instance.h"
 #include "mainwidget.h"
+#include "main/main_app_config.h"
 #include "main/main_session.h"
 #include "main/main_session_settings.h"
 #include "mtproto/mtproto_config.h"
 #include "chat_helpers/message_field.h"
 #include "menu/menu_send.h"
 #include "chat_helpers/emoji_suggestions_widget.h"
+#include "chat_helpers/field_autocomplete.h"
 #include "chat_helpers/tabbed_panel.h"
 #include "chat_helpers/tabbed_selector.h"
 #include "editor/photo_editor_layer_widget.h"
 #include "history/history_drag_area.h"
+#include "history/view/controls/history_view_characters_limit.h"
 #include "history/view/history_view_schedule_box.h"
-#include "core/file_utilities.h"
 #include "core/mime_type.h"
+#include "core/ui_integration.h"
 #include "base/event_filter.h"
 #include "base/call_delayed.h"
 #include "boxes/premium_limits_box.h"
 #include "boxes/premium_preview_box.h"
+#include "boxes/send_credits_box.h"
 #include "ui/effects/scroll_content_shadow.h"
+#include "ui/widgets/fields/number_input.h"
 #include "ui/widgets/checkbox.h"
-#include "ui/widgets/buttons.h"
-#include "ui/widgets/fields/input_field.h"
 #include "ui/widgets/scroll_area.h"
 #include "ui/widgets/popup_menu.h"
-#include "ui/wrap/vertical_layout.h"
-#include "ui/chat/attach/attach_prepare.h"
-#include "ui/chat/attach/attach_send_files_way.h"
 #include "ui/chat/attach/attach_album_preview.h"
 #include "ui/chat/attach/attach_single_file_preview.h"
 #include "ui/chat/attach/attach_single_media_preview.h"
 #include "ui/grouped_layout.h"
+#include "ui/text/text_utilities.h"
 #include "ui/toast/toast.h"
 #include "ui/controls/emoji_button.h"
 #include "ui/painter.h"
+#include "ui/vertical_list.h"
+#include "ui/ui_utility.h"
 #include "lottie/lottie_single_player.h"
+#include "data/data_channel.h"
 #include "data/data_document.h"
 #include "data/data_user.h"
+#include "data/data_peer_values.h" // Data::AmPremiumValue.
 #include "data/data_premium_limits.h"
 #include "data/stickers/data_stickers.h"
 #include "data/stickers/data_custom_emoji.h"
-#include "api/api_common.h"
 #include "window/window_session_controller.h"
 #include "core/application.h"
 #include "core/core_settings.h"
 #include "styles/style_layers.h"
 #include "styles/style_boxes.h"
 #include "styles/style_chat_helpers.h"
-#include "styles/style_menu_icons.h"
 
 #include <QtCore/QMimeData>
 
@@ -109,6 +113,84 @@ rpl::producer<QString> FieldPlaceholder(
 		: tr::lng_photos_comment();
 }
 
+void EditPriceBox(
+		not_null<Ui::GenericBox*> box,
+		not_null<Main::Session*> session,
+		uint64 price,
+		Fn<void(uint64)> apply) {
+	box->setTitle(tr::lng_paid_title());
+	AddSubsectionTitle(
+		box->verticalLayout(),
+		tr::lng_paid_enter_cost(),
+		(st::boxRowPadding - QMargins(
+			st::defaultSubsectionTitlePadding.left(),
+			0,
+			st::defaultSubsectionTitlePadding.right(),
+			0)));
+	const auto limit = session->appConfig().get<int>(
+		u"stars_paid_post_amount_max"_q,
+		10'000);
+	const auto wrap = box->addRow(object_ptr<Ui::FixedHeightWidget>(
+		box,
+		st::editTagField.heightMin));
+	auto owned = object_ptr<Ui::NumberInput>(
+		wrap,
+		st::editTagField,
+		tr::lng_paid_cost_placeholder(),
+		price ? QString::number(price) : QString(),
+		limit);
+	const auto field = owned.data();
+	wrap->widthValue() | rpl::start_with_next([=](int width) {
+		field->move(0, 0);
+		field->resize(width, field->height());
+		wrap->resize(width, field->height());
+	}, wrap->lifetime());
+	field->selectAll();
+	box->setFocusCallback([=] {
+		field->setFocusFast();
+	});
+	const auto about = box->addRow(
+		object_ptr<Ui::FlatLabel>(
+			box,
+			tr::lng_paid_about(
+				lt_link,
+				tr::lng_paid_about_link() | Ui::Text::ToLink(),
+				Ui::Text::WithEntities),
+			st::paidAmountAbout),
+		st::boxRowPadding + QMargins(0, st::sendMediaRowSkip, 0, 0));
+	about->setClickHandlerFilter([=](const auto &...) {
+		Core::App().iv().openWithIvPreferred(
+			session,
+			tr::lng_paid_about_link_url(tr::now));
+		return false;
+	});
+
+	field->paintRequest() | rpl::start_with_next([=](QRect clip) {
+		auto p = QPainter(field);
+		st::paidStarIcon.paint(p, 0, st::paidStarIconTop, field->width());
+	}, field->lifetime());
+
+	const auto save = [=] {
+		const auto now = field->getLastText().toULongLong();
+		if (now > limit) {
+			field->showError();
+			return;
+		}
+		const auto weak = Ui::MakeWeak(box);
+		apply(now);
+		if (const auto strong = weak.data()) {
+			strong->closeBox();
+		}
+	};
+
+	QObject::connect(field, &Ui::NumberInput::submitted, box, save);
+
+	box->addButton(tr::lng_settings_save(), save);
+	box->addButton(tr::lng_cancel(), [=] {
+		box->closeBox();
+	});
+}
+
 } // namespace
 
 SendFilesLimits DefaultLimitsForPeer(not_null<PeerData*> peer) {
@@ -159,7 +241,8 @@ SendFilesBox::Block::Block(
 	int from,
 	int till,
 	Fn<bool()> gifPaused,
-	SendFilesWay way)
+	SendFilesWay way,
+	Fn<bool()> canToggleSpoiler)
 : _items(items)
 , _from(from)
 , _till(till) {
@@ -176,14 +259,16 @@ SendFilesBox::Block::Block(
 			parent.get(),
 			st,
 			my,
-			way);
+			way,
+			std::move(canToggleSpoiler));
 		_preview.reset(preview);
 	} else {
 		const auto media = Ui::SingleMediaPreview::Create(
 			parent,
 			st,
 			gifPaused,
-			first);
+			first,
+			std::move(canToggleSpoiler));
 		if (media) {
 			_isSingleMedia = true;
 			_preview.reset(media);
@@ -259,6 +344,14 @@ rpl::producer<int> SendFilesBox::Block::itemModifyRequest() const {
 	}
 }
 
+rpl::producer<> SendFilesBox::Block::orderUpdated() const {
+	if (_isAlbum) {
+		const auto album = static_cast<Ui::AlbumPreview*>(_preview.get());
+		return album->orderUpdated();
+	}
+	return rpl::never<>();
+}
+
 void SendFilesBox::Block::setSendWay(Ui::SendFilesWay way) {
 	if (!_isAlbum) {
 		if (_isSingleMedia) {
@@ -327,23 +420,35 @@ void SendFilesBox::Block::applyChanges() {
 	}
 }
 
+QImage SendFilesBox::Block::generatePriceTagBackground() const {
+	const auto preview = _preview.get();
+	if (_isAlbum) {
+		const auto album = static_cast<Ui::AlbumPreview*>(preview);
+		return album->generatePriceTagBackground();
+	} else if (_isSingleMedia) {
+		const auto media = static_cast<Ui::SingleMediaPreview*>(preview);
+		return media->generatePriceTagBackground();
+	}
+	return QImage();
+}
+
 SendFilesBox::SendFilesBox(
 	QWidget*,
 	not_null<Window::SessionController*> controller,
 	Ui::PreparedList &&list,
 	const TextWithTags &caption,
-	SendFilesLimits limits,
-	SendFilesCheck check,
+	not_null<PeerData*> toPeer,
 	Api::SendType sendType,
-	SendMenu::Type sendMenuType)
+	SendMenu::Details sendMenuDetails)
 : SendFilesBox(nullptr, {
 	.show = controller->uiShow(),
 	.list = std::move(list),
 	.caption = caption,
-	.limits = limits,
-	.check = check,
+	.captionToPeer = toPeer,
+	.limits = DefaultLimitsForPeer(toPeer),
+	.check = DefaultCheckForPeer(controller, toPeer),
 	.sendType = sendType,
-	.sendMenuType = sendMenuType,
+	.sendMenuDetails = [=] { return sendMenuDetails; },
 }) {
 }
 
@@ -356,7 +461,9 @@ SendFilesBox::SendFilesBox(QWidget*, SendFilesBoxDescriptor &&descriptor)
 , _titleHeight(st::boxTitleHeight)
 , _list(std::move(descriptor.list))
 , _limits(descriptor.limits)
-, _sendMenuType(descriptor.sendMenuType)
+, _sendMenuDetails(prepareSendMenuDetails(descriptor))
+, _sendMenuCallback(prepareSendMenuCallback())
+, _captionToPeer(descriptor.captionToPeer)
 , _check(std::move(descriptor.check))
 , _confirmedCallback(std::move(descriptor.confirmed))
 , _cancelledCallback(std::move(descriptor.cancelled))
@@ -369,6 +476,54 @@ SendFilesBox::SendFilesBox(QWidget*, SendFilesBoxDescriptor &&descriptor)
 	enqueueNextPrepare();
 }
 
+Fn<SendMenu::Details()> SendFilesBox::prepareSendMenuDetails(
+		const SendFilesBoxDescriptor &descriptor) {
+	auto initial = descriptor.sendMenuDetails;
+	return crl::guard(this, [=] {
+		auto result = initial ? initial() : SendMenu::Details();
+		result.spoiler = !hasSpoilerMenu()
+			? SendMenu::SpoilerState::None
+			: allWithSpoilers()
+			? SendMenu::SpoilerState::Enabled
+			: SendMenu::SpoilerState::Possible;
+		const auto way = _sendWay.current();
+		const auto canMoveCaption = _list.canMoveCaption(
+			way.groupFiles() && way.sendImagesAsPhotos(),
+			way.sendImagesAsPhotos()
+		) && _caption && HasSendText(_caption);
+		result.caption = !canMoveCaption
+			? SendMenu::CaptionState::None
+			: _invertCaption
+			? SendMenu::CaptionState::Above
+			: SendMenu::CaptionState::Below;
+		result.price = canChangePrice()
+			? _price.current()
+			: std::optional<uint64>();
+		return result;
+	});
+}
+
+auto SendFilesBox::prepareSendMenuCallback()
+-> Fn<void(MenuAction, MenuDetails)> {
+	return crl::guard(this, [=](MenuAction action, MenuDetails details) {
+		using Type = SendMenu::ActionType;
+		switch (action.type) {
+		case Type::CaptionDown: _invertCaption = false; break;
+		case Type::CaptionUp: _invertCaption = true; break;
+		case Type::SpoilerOn: toggleSpoilers(true); break;
+		case Type::SpoilerOff: toggleSpoilers(false); break;
+		case Type::ChangePrice: changePrice(); break;
+		default:
+			SendMenu::DefaultCallback(
+				_show,
+				sendCallback())(
+					action,
+					details);
+			break;
+		}
+	});
+}
+
 void SendFilesBox::initPreview() {
 	using namespace rpl::mappers;
 
@@ -427,6 +582,7 @@ void SendFilesBox::prepare() {
 	preparePreview();
 	initPreview();
 	SetupShadowsToScrollContent(this, _scroll, _inner->heightValue());
+	setCloseByOutsideClick(false);
 
 	boxClosing() | rpl::start_with_next([=] {
 		if (!_confirmed && _cancelledCallback) {
@@ -533,10 +689,9 @@ void SendFilesBox::refreshButtons() {
 	if (_sendType == Api::SendType::Normal) {
 		SendMenu::SetupMenuAndShortcuts(
 			_send,
-			[=] { return _sendMenuType; },
-			[=] { sendSilent(); },
-			[=] { sendScheduled(); },
-			[=] { sendWhenOnline(); });
+			_show,
+			_sendMenuDetails,
+			_sendMenuCallback);
 	}
 	addButton(tr::lng_cancel(), [=] { closeBox(); });
 	_addFile = addLeftButton(
@@ -548,21 +703,27 @@ void SendFilesBox::refreshButtons() {
 	addMenuButton();
 }
 
-bool SendFilesBox::hasSendMenu() const {
-	return (_sendMenuType != SendMenu::Type::Disabled);
+bool SendFilesBox::hasSendMenu(const MenuDetails &details) const {
+	return (details.type != SendMenu::Type::Disabled)
+		|| (details.spoiler != SendMenu::SpoilerState::None)
+		|| (details.caption != SendMenu::CaptionState::None);
 }
 
 bool SendFilesBox::hasSpoilerMenu() const {
-	const auto allAreVideo = !ranges::any_of(_list.files, [](const auto &f) {
-		using Type = Ui::PreparedFile::Type;
-		return (f.type != Type::Video);
-	});
-	const auto allAreMedia = !ranges::any_of(_list.files, [](const auto &f) {
-		using Type = Ui::PreparedFile::Type;
-		return (f.type != Type::Photo) && (f.type != Type::Video);
-	});
-	return allAreVideo
-		|| (allAreMedia && _sendWay.current().sendImagesAsPhotos());
+	return !hasPrice()
+		&& _list.hasSpoilerMenu(_sendWay.current().sendImagesAsPhotos());
+}
+
+bool SendFilesBox::canChangePrice() const {
+	const auto way = _sendWay.current();
+	const auto broadcast = _captionToPeer
+		? _captionToPeer->asBroadcast()
+		: nullptr;
+	return broadcast
+		&& broadcast->canPostPaidMedia()
+		&& _list.canChangePrice(
+			way.groupFiles() && way.sendImagesAsPhotos(),
+			way.sendImagesAsPhotos());
 }
 
 void SendFilesBox::applyBlockChanges() {
@@ -585,41 +746,139 @@ void SendFilesBox::toggleSpoilers(bool enabled) {
 	}
 }
 
+void SendFilesBox::changePrice() {
+	const auto weak = Ui::MakeWeak(this);
+	const auto session = &_show->session();
+	const auto now = _price.current();
+	_show->show(Box(EditPriceBox, session, now, [=](uint64 price) {
+		if (weak && price != now) {
+			_price = price;
+			refreshPriceTag();
+		}
+	}));
+}
+
+bool SendFilesBox::hasPrice() const {
+	return canChangePrice() && _price.current() > 0;
+}
+
+void SendFilesBox::refreshPriceTag() {
+	const auto resetSpoilers = hasPrice() || _priceTag;
+	if (resetSpoilers) {
+		for (auto &file : _list.files) {
+			file.spoiler = false;
+		}
+		for (auto &block : _blocks) {
+			block.toggleSpoilers(hasPrice());
+		}
+	}
+	if (!hasPrice()) {
+		_priceTag = nullptr;
+		_priceTagBg = QImage();
+	} else if (!_priceTag) {
+		_priceTag = std::make_unique<Ui::RpWidget>(_inner.data());
+		const auto raw = _priceTag.get();
+
+		raw->show();
+		raw->paintRequest() | rpl::start_with_next([=] {
+			if (_priceTagBg.isNull()) {
+				_priceTagBg = preparePriceTagBg(raw->size());
+			}
+			QPainter(raw).drawImage(0, 0, _priceTagBg);
+		}, raw->lifetime());
+
+		const auto session = &_show->session();
+		auto price = _price.value() | rpl::map([=](uint64 amount) {
+			auto result = Ui::Text::Colorized(Ui::CreditsEmoji(session));
+			result.append(Lang::FormatCountDecimal(amount));
+			return result;
+		});
+		auto text = tr::lng_paid_price(
+			lt_price,
+			std::move(price),
+			Ui::Text::WithEntities);
+		const auto label = Ui::CreateChild<Ui::FlatLabel>(
+			raw,
+			QString(),
+			st::paidTagLabel);
+		std::move(text) | rpl::start_with_next([=](TextWithEntities &&text) {
+			label->setMarkedText(text, Core::MarkedTextContext{
+				.session = session,
+				.customEmojiRepaint = [=] { label->update(); },
+			});
+		}, label->lifetime());
+		label->show();
+		label->sizeValue() | rpl::start_with_next([=](QSize size) {
+			const auto inner = QRect(QPoint(), size);
+			const auto rect = inner.marginsAdded(st::paidTagPadding);
+			raw->resize(rect.size());
+			label->move(-rect.topLeft());
+		}, label->lifetime());
+		_inner->sizeValue() | rpl::start_with_next([=](QSize size) {
+			raw->move(
+				(size.width() - raw->width()) / 2,
+				(size.height() - raw->height()) / 2);
+		}, raw->lifetime());
+	} else {
+		_priceTag->raise();
+		_priceTag->update();
+		_priceTagBg = QImage();
+	}
+}
+
+QImage SendFilesBox::preparePriceTagBg(QSize size) const {
+	const auto ratio = style::DevicePixelRatio();
+	const auto outer = _blocks.empty()
+		? size
+		: _inner->widgetAt(0)->geometry().size();
+	auto bg = _blocks.empty()
+		? QImage()
+		: _blocks.front().generatePriceTagBackground();
+	if (bg.isNull()) {
+		bg = QImage(ratio, ratio, QImage::Format_ARGB32_Premultiplied);
+		bg.fill(Qt::black);
+	}
+
+	auto result = QImage(size * ratio, QImage::Format_ARGB32_Premultiplied);
+	result.setDevicePixelRatio(ratio);
+	result.fill(Qt::black);
+	auto p = QPainter(&result);
+	auto hq = PainterHighQualityEnabler(p);
+	p.drawImage(
+		QRect(
+			(size.width() - outer.width()) / 2,
+			(size.height() - outer.height()) / 2,
+			outer.width(),
+			outer.height()),
+		bg);
+	p.fillRect(QRect(QPoint(), size), st::msgDateImgBg);
+	p.end();
+
+	const auto radius = std::min(size.width(), size.height()) / 2;
+	return Images::Round(std::move(result), Images::CornersMask(radius));
+}
+
 void SendFilesBox::addMenuButton() {
-	if (!hasSendMenu() && !hasSpoilerMenu()) {
+	const auto details = _sendMenuDetails();
+	if (!hasSendMenu(details)) {
 		return;
 	}
 
 	const auto top = addTopButton(_st.files.menu);
 	top->setClickedCallback([=] {
 		const auto &tabbed = _st.tabbed;
-		const auto &icons = tabbed.icons;
 		_menu = base::make_unique_q<Ui::PopupMenu>(top, tabbed.menu);
-		if (hasSpoilerMenu()) {
-			const auto spoilered = allWithSpoilers();
-			_menu->addAction(
-				(spoilered
-					? tr::lng_context_disable_spoiler(tr::now)
-					: tr::lng_context_spoiler_effect(tr::now)),
-				[=] { toggleSpoilers(!spoilered); },
-				spoilered ? &icons.menuSpoilerOff : &icons.menuSpoiler);
-			if (hasSendMenu()) {
-				_menu->addSeparator(&tabbed.expandedSeparator);
-			}
-		}
-		if (hasSendMenu()) {
-			SendMenu::FillSendMenu(
-				_menu.get(),
-				_sendMenuType,
-				[=] { sendSilent(); },
-				[=] { sendScheduled(); },
-				[=] { sendWhenOnline(); },
-				&_st.tabbed.icons);
-		}
-		_menu->popup(QCursor::pos());
+		const auto position = QCursor::pos();
+		SendMenu::FillSendMenu(
+			_menu.get(),
+			_show,
+			_sendMenuDetails(),
+			_sendMenuCallback,
+			&_st.tabbed.icons,
+			position);
+		_menu->popup(position);
 		return true;
 	});
-
 }
 
 void SendFilesBox::initSendWay() {
@@ -661,9 +920,8 @@ void SendFilesBox::initSendWay() {
 		for (auto &block : _blocks) {
 			block.setSendWay(value);
 		}
-		if (!hasSendMenu()) {
-			refreshButtons();
-		}
+		refreshButtons();
+		refreshPriceTag();
 		if (was != hidden()) {
 			updateBoxSize();
 			updateControlsGeometry();
@@ -749,7 +1007,8 @@ void SendFilesBox::pushBlock(int from, int till) {
 		from,
 		till,
 		gifPaused,
-		_sendWay.current());
+		_sendWay.current(),
+		[=] { return !hasPrice(); });
 	auto &block = _blocks.back();
 	const auto widget = _inner->add(
 		block.takeWidget(),
@@ -872,12 +1131,18 @@ void SendFilesBox::pushBlock(int from, int till) {
 			st::sendMediaPreviewSize,
 			[=] { refreshAllAfterChanges(from); });
 	}, widget->lifetime());
+
+	block.orderUpdated() | rpl::start_with_next([=]{
+		if (_priceTag) {
+			_priceTagBg = QImage();
+			_priceTag->update();
+		}
+	}, widget->lifetime());
 }
 
 void SendFilesBox::refreshControls(bool initial) {
-	if (initial || !hasSendMenu()) {
-		refreshButtons();
-	}
+	refreshButtons();
+	refreshPriceTag();
 	refreshTitleText();
 	updateSendWayControls();
 	updateCaptionPlaceholder();
@@ -996,17 +1261,23 @@ void SendFilesBox::updateSendWayControls() {
 }
 
 void SendFilesBox::setupCaption() {
-	const auto allow = [=](const auto &) {
-		return (_limits & SendFilesAllow::EmojiWithoutPremium);
+	const auto allow = [=](not_null<DocumentData*> emoji) {
+		return _captionToPeer
+			? Data::AllowEmojiWithoutPremium(_captionToPeer, emoji)
+			: (_limits & SendFilesAllow::EmojiWithoutPremium);
 	};
 	const auto show = _show;
-	InitMessageFieldHandlers(
-		&show->session(),
-		show,
-		_caption.data(),
-		[=] { return show->paused(Window::GifPauseReason::Layer); },
-		allow,
-		&_st.files.caption);
+	InitMessageFieldHandlers({
+		.session = &show->session(),
+		.show = show,
+		.field = _caption.data(),
+		.customEmojiPaused = [=] {
+			return show->paused(Window::GifPauseReason::Layer);
+		},
+		.allowPremiumEmoji = allow,
+		.fieldStyle = &_st.files.caption,
+	});
+	setupCaptionAutocomplete();
 	Ui::Emoji::SuggestionsController::Init(
 		getDelegate()->outerContainer(),
 		_caption,
@@ -1058,6 +1329,92 @@ void SendFilesBox::setupCaption() {
 
 	updateCaptionPlaceholder();
 	setupEmojiPanel();
+
+	rpl::single(rpl::empty_value()) | rpl::then(
+		_caption->changes()
+	) | rpl::start_with_next([=] {
+		checkCharsLimitation();
+	}, _caption->lifetime());
+}
+
+void SendFilesBox::setupCaptionAutocomplete() {
+	if (!_captionToPeer || !_caption) {
+		return;
+	}
+	const auto parent = getDelegate()->outerContainer();
+	ChatHelpers::InitFieldAutocomplete(_autocomplete, {
+		.parent = parent,
+		.show = _show,
+		.field = _caption.data(),
+		.peer = _captionToPeer,
+		.features = [=] {
+			auto result = ChatHelpers::ComposeFeatures();
+			result.autocompleteCommands = false;
+			result.suggestStickersByEmoji = false;
+			return result;
+		},
+		.sendMenuDetails = _sendMenuDetails,
+	});
+	const auto raw = _autocomplete.get();
+	const auto scheduled = std::make_shared<bool>();
+	const auto recountPostponed = [=] {
+		if (*scheduled) {
+			return;
+		}
+		*scheduled = true;
+		Ui::PostponeCall(raw, [=] {
+			*scheduled = false;
+
+			auto field = Ui::MapFrom(parent, this, _caption->geometry());
+			_autocomplete->setBoundings(QRect(
+				field.x() - _caption->x(),
+				st::defaultBox.margin.top(),
+				width(),
+				(field.y()
+					+ _st.files.caption.textMargins.top()
+					+ _st.files.caption.placeholderShift
+					+ _st.files.caption.placeholderFont->height
+					- st::defaultBox.margin.top())));
+		});
+	};
+	for (auto w = (QWidget*)_caption.data(); w; w = w->parentWidget()) {
+		base::install_event_filter(raw, w, [=](not_null<QEvent*> e) {
+			if (e->type() == QEvent::Move || e->type() == QEvent::Resize) {
+				recountPostponed();
+			}
+			return base::EventFilterResult::Continue;
+		});
+		if (w == parent) {
+			break;
+		}
+	}
+}
+
+void SendFilesBox::checkCharsLimitation() {
+	const auto limits = Data::PremiumLimits(&_show->session());
+	const auto caption = (_caption && !_caption->isHidden())
+		? _caption->getTextWithAppliedMarkdown()
+		: TextWithTags();
+	const auto remove = caption.text.size() - limits.captionLengthCurrent();
+	if ((remove > 0) && _emojiToggle) {
+		if (!_charsLimitation) {
+			_charsLimitation = base::make_unique_q<CharactersLimitLabel>(
+				this,
+				_emojiToggle.data(),
+				style::al_top);
+			_charsLimitation->show();
+			Data::AmPremiumValue(
+				&_show->session()
+			) | rpl::start_with_next([=] {
+				checkCharsLimitation();
+			}, _charsLimitation->lifetime());
+		}
+		_charsLimitation->setLeft(remove);
+	} else {
+		if (_charsLimitation) {
+			_charsLimitation = nullptr;
+		}
+	}
 }
 
 void SendFilesBox::setupEmojiPanel() {
@@ -1076,7 +1433,6 @@ void SendFilesBox::setupEmojiPanel() {
 					.level = Window::GifPauseReason::Layer,
 					.mode = ChatHelpers::TabbedSelector::Mode::EmojiOnly,
 					.features = {
-						.megagroupSet = false,
 						.stickersSettings = false,
 						.openStickerSets = false,
 					},
@@ -1087,6 +1443,7 @@ void SendFilesBox::setupEmojiPanel() {
 		st::emojiPanMinHeight / 2,
 		st::emojiPanMinHeight);
 	_emojiPanel->hide();
+	_emojiPanel->selector()->setCurrentPeer(_captionToPeer);
 	_emojiPanel->selector()->setAllowEmojiWithoutPremium(
 		_limits & SendFilesAllow::EmojiWithoutPremium);
 	_emojiPanel->selector()->emojiChosen(
@@ -1099,8 +1456,12 @@ void SendFilesBox::setupEmojiPanel() {
 		if (info
 			&& info->setType == Data::StickersType::Emoji
 			&& !_show->session().premium()
-			&& !(_limits & SendFilesAllow::EmojiWithoutPremium)) {
-			ShowPremiumPreviewBox(_show, PremiumPreview::AnimatedEmoji);
+			&& !(_captionToPeer
+				? Data::AllowEmojiWithoutPremium(
+					_captionToPeer,
+					data.document)
+				: (_limits & SendFilesAllow::EmojiWithoutPremium))) {
+			ShowPremiumPreviewBox(_show, PremiumFeature::AnimatedEmoji);
 		} else {
 			Data::InsertCustomEmoji(_caption.data(), data.document);
 		}
@@ -1344,6 +1705,14 @@ void SendFilesBox::updateControlsGeometry() {
 	_scroll->move(0, _titleHeight.current());
 }
 
+void SendFilesBox::showFinished() {
+	if (const auto raw = _autocomplete.get()) {
+		InvokeQueued(raw, [=] {
+			raw->raise();
+		});
+	}
+}
+
 void SendFilesBox::setInnerFocus() {
 	if (_caption && !_caption->isHidden()) {
 		_caption->setFocusFast();
@@ -1390,7 +1759,13 @@ void SendFilesBox::send(
 	if ((_sendType == Api::SendType::Scheduled
 		|| _sendType == Api::SendType::ScheduledToUser)
 		&& !options.scheduled) {
-		return sendScheduled();
+		auto child = _sendMenuDetails();
+		child.spoiler = SendMenu::SpoilerState::None;
+		child.caption = SendMenu::CaptionState::None;
+		child.price = std::nullopt;
+		return SendMenu::DefaultCallback(_show, sendCallback())(
+			{ .type = SendMenu::ActionType::Schedule },
+			child);
 	}
 	if (_preparing) {
 		_whenReadySend = [=] {
@@ -1418,6 +1793,13 @@ void SendFilesBox::send(
 		if (!validateLength(caption.text)) {
 			return;
 		}
+		options.invertCaption = _invertCaption;
+		options.price = hasPrice() ? _price.current() : 0;
+		if (options.price > 0) {
+			for (auto &file : _list.files) {
+				file.spoiler = false;
+			}
+		}
 		_confirmedCallback(
 			std::move(_list),
 			_sendWay.current(),
@@ -1428,20 +1810,10 @@ void SendFilesBox::send(
 	closeBox();
 }
 
-void SendFilesBox::sendSilent() {
-	send({ .silent = true });
-}
-
-void SendFilesBox::sendScheduled() {
-	const auto type = (_sendType == Api::SendType::ScheduledToUser)
-		? SendMenu::Type::ScheduledToUser
-		: _sendMenuType;
-	const auto callback = [=](Api::SendOptions options) { send(options); };
-	_show->showBox(HistoryView::PrepareScheduleBox(this, type, callback));
-}
-
-void SendFilesBox::sendWhenOnline() {
-	send(Api::DefaultSendWhenOnlineOptions());
+Fn<void(Api::SendOptions)> SendFilesBox::sendCallback() {
+	return crl::guard(this, [=](Api::SendOptions options) {
+		send(options, false);
+	});
 }
 
 SendFilesBox::~SendFilesBox() = default;
