@@ -17,9 +17,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "main/main_app_config_values.h"
 #include "main/main_session.h"
 #include "mtproto/sender.h"
-#include "settings/settings_common.h"
 #include "ui/layers/generic_box.h"
 #include "ui/painter.h"
+#include "ui/vertical_list.h"
 #include "ui/text/text_utilities.h"
 #include "ui/text/text_variant.h"
 #include "ui/toast/toast.h"
@@ -30,21 +30,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/wrap/slide_wrap.h"
 #include "styles/style_layers.h"
 #include "styles/style_boxes.h"
-#include "styles/style_settings.h"
 
 namespace {
 
-[[nodiscard]] TextWithEntities PurchaseAvailableText() {
-	constexpr auto kUsernameAuction = "auction";
-	return tr::lng_username_purchase_available(
-		tr::now,
-		lt_link,
-		Ui::Text::Link(
-			'@' + QString(kUsernameAuction),
-			u"https://t.me/"_q + kUsernameAuction),
-		Ui::Text::RichLangValue);
-}
-
 class UsernameEditor final : public Ui::RpWidget {
 public:
 	UsernameEditor(not_null<Ui::RpWidget*>, not_null<PeerData*> peer);
@@ -269,9 +257,8 @@ void UsernameEditor::checkInfoPurchaseAvailable() {
 	_username->showError();
 	_errorText = u".bad."_q;
 
-	_checkInfoChanged.fire({
-		.type = UsernameCheckInfo::Type::PurchaseAvailable,
-	});
+	_checkInfoChanged.fire(
+		UsernameCheckInfo::PurchaseAvailable(_checkUsername, _peer));
 }
 
 void UsernameEditor::updateFail(const QString &error) {
@@ -369,7 +356,7 @@ void UsernamesBox(
 			container,
 			std::move(description),
 			st::boxDividerLabel),
-		st::settingsDividerLabelPadding));
+		st::defaultBoxDividerLabelPadding));
 
 	const auto list = box->addRow(
 		object_ptr<UsernamesList>(
@@ -409,11 +396,11 @@ void AddUsernameCheckLabel(
 	const auto skip = (st::usernameSkip - st.style.font->height) / 4;
 
 	auto wrapped = object_ptr<Ui::VerticalLayout>(container);
-	Settings::AddSkip(wrapped, skip);
+	Ui::AddSkip(wrapped, skip);
 	const auto label = wrapped->add(object_ptr<Ui::FlatLabel>(wrapped, st));
-	Settings::AddSkip(wrapped, skip);
+	Ui::AddSkip(wrapped, skip);
 
-	Settings::AddSkip(container, skip);
+	Ui::AddSkip(container, skip);
 	container->add(
 		object_ptr<Ui::FollowSlideWrap<Ui::VerticalLayout>>(
 			container,
@@ -425,9 +412,7 @@ void AddUsernameCheckLabel(
 		container->widthValue()
 	) | rpl::start_with_next([=](const UsernameCheckInfo &info, int w) {
 		using Type = UsernameCheckInfo::Type;
-		label->setMarkedText((info.type == Type::PurchaseAvailable)
-			? PurchaseAvailableText()
-			: info.text);
+		label->setMarkedText(info.text);
 		const auto &color = (info.type == Type::Good)
 			? st::boxTextFgGood
 			: (info.type == Type::Error)
@@ -436,5 +421,27 @@ void AddUsernameCheckLabel(
 		label->setTextColorOverride(color->c);
 		label->resizeToWidth(w - padding.left() - padding.right());
 	}, label->lifetime());
-	Settings::AddSkip(container, skip);
+	Ui::AddSkip(container, skip);
+}
+
+UsernameCheckInfo UsernameCheckInfo::PurchaseAvailable(
+		const QString &username,
+		not_null<PeerData*> peer) {
+	if (const auto fragmentLink = AppConfig::FragmentLink(&peer->session())) {
+		return {
+			.type = UsernameCheckInfo::Type::Default,
+			.text = tr::lng_username_purchase_available(
+				tr::now,
+				lt_link,
+				Ui::Text::Link(
+					tr::lng_username_purchase_available_link(tr::now),
+					(*fragmentLink) + u"/username/"_q + username),
+				Ui::Text::RichLangValue),
+		};
+	} else {
+		return {
+			.type = UsernameCheckInfo::Type::Error,
+			.text = { u"INTERNAL_SERVER_ERROR"_q },
+		};
+	}
 }
