@@ -7,22 +7,25 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "data/data_user.h"
 
+#include "api/api_credits.h"
+#include "api/api_sensitive_content.h"
+#include "api/api_statistics.h"
 #include "storage/localstorage.h"
 #include "storage/storage_user_photos.h"
 #include "main/main_session.h"
+#include "data/business/data_business_common.h"
+#include "data/business/data_business_info.h"
+#include "data/components/credits.h"
 #include "data/data_session.h"
 #include "data/data_changes.h"
 #include "data/data_peer_bot_command.h"
 #include "data/data_photo.h"
 #include "data/data_stories.h"
-#include "data/data_emoji_statuses.h"
-#include "data/data_user_names.h"
 #include "data/data_wall_paper.h"
 #include "data/notify/data_notify_settings.h"
 #include "history/history.h"
 #include "api/api_peer_photo.h"
 #include "apiwrap.h"
-#include "ui/text/text_options.h"
 #include "lang/lang_keys.h"
 #include "styles/style_chat.h"
 
@@ -37,11 +40,36 @@ using UpdateFlag = Data::PeerUpdate::Flag;
 
 BotInfo::BotInfo() = default;
 
+Data::LastseenStatus LastseenFromMTP(
+		const MTPUserStatus &status,
+		Data::LastseenStatus currentStatus) {
+	return status.match([](const MTPDuserStatusEmpty &data) {
+		return Data::LastseenStatus::LongAgo();
+	}, [&](const MTPDuserStatusRecently &data) {
+		return currentStatus.isLocalOnlineValue()
+			? Data::LastseenStatus::OnlineTill(
+				currentStatus.onlineTill(),
+				true,
+				data.is_by_me())
+			: Data::LastseenStatus::Recently(data.is_by_me());
+	}, [](const MTPDuserStatusLastWeek &data) {
+		return Data::LastseenStatus::WithinWeek(data.is_by_me());
+	}, [](const MTPDuserStatusLastMonth &data) {
+		return Data::LastseenStatus::WithinMonth(data.is_by_me());
+	}, [](const MTPDuserStatusOnline& data) {
+		return Data::LastseenStatus::OnlineTill(data.vexpires().v);
+	}, [](const MTPDuserStatusOffline &data) {
+		return Data::LastseenStatus::OnlineTill(data.vwas_online().v);
+	});
+}
+
 UserData::UserData(not_null<Data::Session*> owner, PeerId id)
 : PeerData(owner, id)
 , _flags((id == owner->session().userPeerId()) ? Flag::Self : Flag(0)) {
 }
 
+UserData::~UserData() = default;
+
 bool UserData::canShareThisContact() const {
 	return canShareThisContactFast()
 		|| !owner().findContactPhone(peerToUser(id)).isEmpty();
@@ -57,6 +85,19 @@ void UserData::setIsContact(bool is) {
 	}
 }
 
+Data::LastseenStatus UserData::lastseen() const {
+	return _lastseen;
+}
+
+bool UserData::updateLastseen(Data::LastseenStatus value) {
+	if (_lastseen == value) {
+		return false;
+	}
+	_lastseen = value;
+	owner().maybeStopWatchForOffline(this);
+	return true;
+}
+
 // see Serialize::readPeer as well
 void UserData::setPhoto(const MTPUserProfilePhoto &photo) {
 	photo.match([&](const MTPDuserProfilePhoto &data) {
@@ -75,36 +116,14 @@ void UserData::setPhoto(const MTPUserProfilePhoto &photo) {
 	});
 }
 
-void UserData::setEmojiStatus(const MTPEmojiStatus &status) {
-	const auto parsed = Data::ParseEmojiStatus(status);
-	setEmojiStatus(parsed.id, parsed.until);
-}
-
-void UserData::setEmojiStatus(DocumentId emojiStatusId, TimeId until) {
-	if (_emojiStatusId != emojiStatusId) {
-		_emojiStatusId = emojiStatusId;
-		session().changes().peerUpdated(this, UpdateFlag::EmojiStatus);
-	}
-	owner().emojiStatuses().registerAutomaticClear(this, until);
-}
-
-DocumentId UserData::emojiStatusId() const {
-	return _emojiStatusId;
-}
-
 auto UserData::unavailableReasons() const
 -> const std::vector<Data::UnavailableReason> & {
 	return _unavailableReasons;
 }
 
-void UserData::setUnavailableReasons(
+void UserData::setUnavailableReasonsList(
 		std::vector<Data::UnavailableReason> &&reasons) {
-	if (_unavailableReasons != reasons) {
-		_unavailableReasons = std::move(reasons);
-		session().changes().peerUpdated(
-			this,
-			UpdateFlag::UnavailableReason);
-	}
+	_unavailableReasons = std::move(reasons);
 }
 
 void UserData::setCommonChatsCount(int count) {
@@ -114,6 +133,17 @@ void UserData::setCommonChatsCount(int count) {
 	}
 }
 
+int UserData::peerGiftsCount() const {
+	return _peerGiftsCount;
+}
+
+void UserData::setPeerGiftsCount(int count) {
+	if (_peerGiftsCount != count) {
+		_peerGiftsCount = count;
+		session().changes().peerUpdated(this, UpdateFlag::PeerGifts);
+	}
+}
+
 bool UserData::hasPrivateForwardName() const {
 	return !_privateForwardName.isEmpty();
 }
@@ -158,6 +188,40 @@ void UserData::setStoriesState(StoriesState state) {
 	}
 }
 
+const Data::BusinessDetails &UserData::businessDetails() const {
+	static const auto empty = Data::BusinessDetails();
+	return _businessDetails ? *_businessDetails : empty;
+}
+
+void UserData::setBusinessDetails(Data::BusinessDetails details) {
+	details.hours = details.hours.normalized();
+	if ((!details && !_businessDetails)
+		|| (details && _businessDetails && details == *_businessDetails)) {
+		return;
+	}
+	_businessDetails = details
+		? std::make_unique<Data::BusinessDetails>(std::move(details))
+		: nullptr;
+	session().changes().peerUpdated(this, UpdateFlag::BusinessDetails);
+}
+
+ChannelId UserData::personalChannelId() const {
+	return _personalChannelId;
+}
+
+MsgId UserData::personalChannelMessageId() const {
+	return _personalChannelMessageId;
+}
+
+void UserData::setPersonalChannel(ChannelId channelId, MsgId messageId) {
+	if (_personalChannelId != channelId
+		|| _personalChannelMessageId != messageId) {
+		_personalChannelId = channelId;
+		_personalChannelMessageId = messageId;
+		session().changes().peerUpdated(this, UpdateFlag::PersonalChannel);
+	}
+}
+
 void UserData::setName(const QString &newFirstName, const QString &newLastName, const QString &newPhoneName, const QString &newUsername) {
 	bool changeName = !newFirstName.isEmpty() || !newLastName.isEmpty();
 
@@ -277,7 +341,11 @@ void UserData::setBotInfo(const MTPBotInfo &info) {
 			d.vmenu_button());
 		botInfo->inited = true;
 
-		if (changedCommands || changedButton) {
+		const auto privacy = qs(d.vprivacy_policy_url().value_or_empty());
+		const auto privacyChanged = (botInfo->privacyPolicyUrl != privacy);
+		botInfo->privacyPolicyUrl = privacy;
+
+		if (changedCommands || changedButton || privacyChanged) {
 			owner().botCommandsChanged(this);
 		}
 	} break;
@@ -291,11 +359,14 @@ void UserData::setNameOrPhone(const QString &newNameOrPhone) {
 void UserData::madeAction(TimeId when) {
 	if (isBot() || isServiceUser() || when <= 0) {
 		return;
-	} else if (onlineTill <= 0 && -onlineTill < when) {
-		onlineTill = -when - kSetOnlineAfterActivity;
-		session().changes().peerUpdated(this, UpdateFlag::OnlineStatus);
-	} else if (onlineTill > 0 && onlineTill < when + 1) {
-		onlineTill = when + kSetOnlineAfterActivity;
+	}
+	const auto till = lastseen().onlineTill();
+	if (till < when + 1
+		&& updateLastseen(
+			Data::LastseenStatus::OnlineTill(
+				when + kSetOnlineAfterActivity,
+				!till || lastseen().isLocalOnlineValue(),
+				lastseen().isHiddenByMe()))) {
 		session().changes().peerUpdated(this, UpdateFlag::OnlineStatus);
 	}
 }
@@ -371,12 +442,28 @@ bool UserData::hasStoriesHidden() const {
 	return (flags() & UserDataFlag::StoriesHidden);
 }
 
-bool UserData::canAddContact() const {
-	return canShareThisContact() && !isContact();
+bool UserData::someRequirePremiumToWrite() const {
+	return (flags() & UserDataFlag::SomeRequirePremiumToWrite);
 }
 
-bool UserData::canReceiveGifts() const {
-	return flags() & UserDataFlag::CanReceiveGifts;
+bool UserData::meRequiresPremiumToWrite() const {
+	return !isSelf() && (flags() & UserDataFlag::MeRequiresPremiumToWrite);
+}
+
+bool UserData::requirePremiumToWriteKnown() const {
+	return (flags() & UserDataFlag::RequirePremiumToWriteKnown);
+}
+
+bool UserData::canSendIgnoreRequirePremium() const {
+	return !isInaccessible() && !isRepliesChat() && !isVerifyCodes();
+}
+
+bool UserData::readDatesPrivate() const {
+	return (flags() & UserDataFlag::ReadDatesPrivate);
+}
+
+bool UserData::canAddContact() const {
+	return canShareThisContact() && !isContact();
 }
 
 bool UserData::canShareThisContactFast() const {
@@ -388,13 +475,17 @@ QString UserData::username() const {
 }
 
 QString UserData::editableUsername() const {
-	return _username.editableUsername();;
+	return _username.editableUsername();
 }
 
 const std::vector<QString> &UserData::usernames() const {
 	return _username.usernames();
 }
 
+bool UserData::isUsernameEditable(QString username) const {
+	return _username.isEditable(username);
+}
+
 const QString &UserData::phone() const {
 	return _phone;
 }
@@ -422,6 +513,34 @@ void UserData::setCallsStatus(CallsStatus callsStatus) {
 	}
 }
 
+
+Data::Birthday UserData::birthday() const {
+	return _birthday;
+}
+
+void UserData::setBirthday(Data::Birthday value) {
+	if (_birthday != value) {
+		_birthday = value;
+		session().changes().peerUpdated(this, UpdateFlag::Birthday);
+
+		if (isSelf()) {
+			session().api().sensitiveContent().reload(true);
+		}
+	}
+}
+
+void UserData::setBirthday(const tl::conditional<MTPBirthday> &value) {
+	if (!value) {
+		setBirthday(Data::Birthday());
+	} else {
+		const auto &data = value->data();
+		setBirthday(Data::Birthday(
+			data.vday().v,
+			data.vmonth().v,
+			data.vyear().value_or_empty()));
+	}
+}
+
 bool UserData::hasCalls() const {
 	return (callsStatus() != CallsStatus::Disabled)
 		&& (callsStatus() != CallsStatus::Unknown);
@@ -452,7 +571,7 @@ void ApplyUserUpdate(not_null<UserData*> user, const MTPDuserFull &update) {
 			));
 		}
 	}
-	user->setSettings(update.vsettings());
+	user->setBarSettings(update.vsettings());
 	user->owner().notifySettings().apply(user, update.vnotify_settings());
 
 	user->setMessagesTTL(update.vttl_period().value_or_empty());
@@ -464,24 +583,29 @@ void ApplyUserUpdate(not_null<UserData*> user, const MTPDuserFull &update) {
 	if (const auto pinned = update.vpinned_msg_id()) {
 		SetTopPinnedMessageId(user, pinned->v);
 	}
-	const auto canReceiveGifts = (update.vflags().v
-			& MTPDuserFull::Flag::f_premium_gifts)
-		&& update.vpremium_gifts();
 	using Flag = UserDataFlag;
 	const auto mask = Flag::Blocked
 		| Flag::HasPhoneCalls
 		| Flag::PhoneCallsPrivate
-		| Flag::CanReceiveGifts
 		| Flag::CanPinMessages
-		| Flag::VoiceMessagesForbidden;
+		| Flag::VoiceMessagesForbidden
+		| Flag::ReadDatesPrivate
+		| Flag::RequirePremiumToWriteKnown
+		| Flag::MeRequiresPremiumToWrite;
 	user->setFlags((user->flags() & ~mask)
-		| (update.is_phone_calls_private() ? Flag::PhoneCallsPrivate : Flag())
+		| (update.is_phone_calls_private()
+			? Flag::PhoneCallsPrivate
+			: Flag())
 		| (update.is_phone_calls_available() ? Flag::HasPhoneCalls : Flag())
-		| (canReceiveGifts ? Flag::CanReceiveGifts : Flag())
 		| (update.is_can_pin_message() ? Flag::CanPinMessages : Flag())
 		| (update.is_blocked() ? Flag::Blocked : Flag())
 		| (update.is_voice_messages_forbidden()
 			? Flag::VoiceMessagesForbidden
+			: Flag())
+		| (update.is_read_dates_private() ? Flag::ReadDatesPrivate : Flag())
+		| Flag::RequirePremiumToWriteKnown
+		| (update.is_contact_require_premium()
+			? Flag::MeRequiresPremiumToWrite
 			: Flag()));
 	user->setIsBlocked(update.is_blocked());
 	user->setCallsStatus(update.is_phone_calls_private()
@@ -491,6 +615,7 @@ void ApplyUserUpdate(not_null<UserData*> user, const MTPDuserFull &update) {
 		: UserData::CallsStatus::Disabled);
 	user->setAbout(qs(update.vabout().value_or_empty()));
 	user->setCommonChatsCount(update.vcommon_chats_count().v);
+	user->setPeerGiftsCount(update.vstargifts_count().value_or_empty());
 	user->checkFolder(update.vfolder_id().value_or_empty());
 	user->setThemeEmoji(qs(update.vtheme_emoticon().value_or_empty()));
 	user->setTranslationDisabled(update.is_translations_disabled());
@@ -513,15 +638,61 @@ void ApplyUserUpdate(not_null<UserData*> user, const MTPDuserFull &update) {
 				user,
 				Data::PeerUpdate::Flag::Rights);
 		}
+		if (info->canEditInformation) {
+			const auto id = user->id;
+			const auto weak = base::make_weak(&user->session());
+			const auto creditsLoadLifetime
+				= std::make_shared<rpl::lifetime>();
+			const auto creditsLoad
+				= creditsLoadLifetime->make_state<Api::CreditsStatus>(user);
+			creditsLoad->request({}, [=](Data::CreditsStatusSlice slice) {
+				if (const auto strong = weak.get()) {
+					strong->credits().apply(id, slice.balance);
+					creditsLoadLifetime->destroy();
+				}
+			});
+			const auto currencyLoadLifetime
+				= std::make_shared<rpl::lifetime>();
+			const auto currencyLoad
+				= currencyLoadLifetime->make_state<Api::EarnStatistics>(user);
+			currencyLoad->request(
+			) | rpl::start_with_error_done([=](const QString &error) {
+				currencyLoadLifetime->destroy();
+			}, [=] {
+				if (const auto strong = weak.get()) {
+					strong->credits().applyCurrency(
+						id,
+						currencyLoad->data().currentBalance);
+					currencyLoadLifetime->destroy();
+				}
+			}, *currencyLoadLifetime);
+		}
 	}
 
 	if (const auto paper = update.vwallpaper()) {
 		user->setWallPaper(
-			Data::WallPaper::Create(&user->session(), *paper));
+			Data::WallPaper::Create(&user->session(), *paper),
+			update.is_wallpaper_overridden());
 	} else {
 		user->setWallPaper({});
 	}
 
+	user->setBusinessDetails(FromMTP(
+		&user->owner(),
+		update.vbusiness_work_hours(),
+		update.vbusiness_location(),
+		update.vbusiness_intro()));
+	user->setBirthday(update.vbirthday());
+	user->setPersonalChannel(
+		update.vpersonal_channel_id().value_or_empty(),
+		update.vpersonal_channel_message().value_or_empty());
+	if (user->isSelf()) {
+		user->owner().businessInfo().applyAwaySettings(
+			FromMTP(&user->owner(), update.vbusiness_away_message()));
+		user->owner().businessInfo().applyGreetingSettings(
+			FromMTP(&user->owner(), update.vbusiness_greeting_message()));
+	}
+
 	user->owner().stories().apply(user, update.vstories());
 
 	user->fullUpdated();
