@@ -251,9 +251,9 @@ void Pip::RendererGL::createShadowTexture() {
 	const auto size = 2 * st::callShadow.topLeft.size()
 		+ QSize(st::roundRadiusLarge, st::roundRadiusLarge);
 	auto image = QImage(
-		size * cIntRetinaFactor(),
+		size * style::DevicePixelRatio(),
 		QImage::Format_ARGB32_Premultiplied);
-	image.setDevicePixelRatio(cRetinaFactor());
+	image.setDevicePixelRatio(style::DevicePixelRatio());
 	image.fill(Qt::transparent);
 	{
 		auto p = QPainter(&image);
@@ -269,9 +269,10 @@ void Pip::RendererGL::createShadowTexture() {
 void Pip::RendererGL::paint(
 		not_null<QOpenGLWidget*> widget,
 		QOpenGLFunctions &f) {
-	const auto factor = widget->devicePixelRatio();
+	const auto factor = widget->devicePixelRatioF();
 	if (_factor != factor) {
 		_factor = factor;
+		_ifactor = int(std::ceil(_factor));
 		_controlsImage.invalidate();
 	}
 	_blendingEnabled = false;
@@ -426,13 +427,14 @@ void Pip::RendererGL::paintTransformedContent(
 		(geometry.outer.height() - geometry.inner.y()) * yscale,
 	};
 
+	_contentBuffer->bind();
 	_contentBuffer->write(0, coords, sizeof(coords));
 
 	const auto rgbaFrame = _chromaSize.isEmpty();
 	_f->glActiveTexture(rgbaFrame ? GL_TEXTURE1 : GL_TEXTURE3);
 	_shadowImage.bind(*_f);
 
-	const auto globalFactor = cIntRetinaFactor();
+	const auto globalFactor = style::DevicePixelRatio();
 	const auto fadeAlpha = st::radialBg->c.alphaF() * geometry.fade;
 	const auto roundRect = transformRect(RoundingRect(geometry));
 	program->setUniformValue("roundRect", Uniform(roundRect));
@@ -587,6 +589,7 @@ void Pip::RendererGL::paintButton(
 		iconOverRect.texture.right(), iconOverRect.texture.top(),
 		iconOverRect.texture.left(), iconOverRect.texture.top(),
 	};
+	_contentBuffer->bind();
 	_contentBuffer->write(
 		offset * 4 * sizeof(GLfloat),
 		coords,
@@ -667,10 +670,10 @@ void Pip::RendererGL::validateControls() {
 		fullHeight += 2 * meta.icon->height();
 	}
 	auto image = QImage(
-		QSize(maxWidth, fullHeight) * _factor,
+		QSize(maxWidth, fullHeight) * _ifactor,
 		QImage::Format_ARGB32_Premultiplied);
 	image.fill(Qt::transparent);
-	image.setDevicePixelRatio(_factor);
+	image.setDevicePixelRatio(_ifactor);
 	{
 		auto p = QPainter(&image);
 		auto index = 0;
@@ -678,8 +681,8 @@ void Pip::RendererGL::validateControls() {
 		const auto paint = [&](not_null<const style::icon*> icon) {
 			icon->paint(p, 0, height, maxWidth);
 			_controlsTextures[index++] = QRect(
-				QPoint(0, height) * _factor,
-				icon->size() * _factor);
+				QPoint(0, height) * _ifactor,
+				icon->size() * _ifactor);
 			height += icon->height();
 		};
 		for (const auto &meta : metas) {
@@ -702,7 +705,7 @@ void Pip::RendererGL::paintUsingRaster(
 		int bufferOffset,
 		bool transparent) {
 	auto raster = image.takeImage();
-	const auto size = rect.size() * _factor;
+	const auto size = rect.size() * _ifactor;
 	if (raster.width() < size.width() || raster.height() < size.height()) {
 		raster = QImage(size, QImage::Format_ARGB32_Premultiplied);
 		raster.setDevicePixelRatio(_factor);
@@ -711,8 +714,8 @@ void Pip::RendererGL::paintUsingRaster(
 				|| raster.height() > size.height())) {
 			raster.fill(Qt::transparent);
 		}
-	} else if (raster.devicePixelRatio() != _factor) {
-		raster.setDevicePixelRatio(_factor);
+	} else if (raster.devicePixelRatio() != _ifactor) {
+		raster.setDevicePixelRatio(_ifactor);
 	}
 
 	if (transparent) {
@@ -740,6 +743,7 @@ void Pip::RendererGL::paintUsingRaster(
 		geometry.left(), geometry.bottom(),
 		textured.texture.left(), textured.texture.top(),
 	};
+	_contentBuffer->bind();
 	_contentBuffer->write(
 		bufferOffset * 4 * sizeof(GLfloat),
 		coords,
