@@ -4,7 +4,7 @@
 // For license and copyright information please follow this link:
 // https://github.com/desktop-app/legal/blob/master/LEGAL
 //
-#include "catch.hpp"
+#include <catch.hpp>
 
 #include "base/index_based_iterator.h"
 
