@@ -7,6 +7,7 @@
 #include "base/platform/win/base_file_utilities_win.h"
 
 #include "base/platform/win/base_windows_safe_library.h"
+#include "base/platform/win/base_windows_shlobj_h.h"
 #include "base/algorithm.h"
 
 #include <QtCore/QString>
@@ -14,8 +15,8 @@
 
 #include <array>
 #include <string>
+#include <shellapi.h>
 #include <Shlwapi.h>
-#include <shlobj.h>
 #include <RestartManager.h>
 #include <io.h>
 
@@ -181,7 +182,6 @@ bool CloseProcesses(const QString &filename) {
 		return false;
 	}
 
-	auto result = BOOL(FALSE);
 	auto session = DWORD();
 	auto sessionKey = std::wstring(CCH_RM_SESSION_KEY + 1, wchar_t(0));
 	auto error = RmStartSession(&session, 0, sessionKey.data());
