@@ -126,6 +126,12 @@ RpWidget *VerticalLayout::insertChild(
 		_rows.insert(
 			begin(_rows) + atPosition,
 			{ std::move(child), margin });
+		auto availRowWidth = widthNoMargins()
+			- margin.left()
+			- margin.right();
+		if (availRowWidth > 0) {
+			weak->resizeToNaturalWidth(availRowWidth);
+		}
 		weak->heightValue(
 		) | rpl::start_with_next_done([=] {
 			if (!_inResize) {
@@ -198,4 +204,10 @@ void VerticalLayout::removeChild(RpWidget *child) {
 	resize(width(), margins.top() + top + margins.bottom());
 }
 
+void VerticalLayout::clear() {
+	while (!_rows.empty()) {
+		removeChild(_rows.front().widget.data());
+	}
+}
+
 } // namespace Ui
